@@ -6,8 +6,8 @@
 # :title: Diakonos
 #
 # Author:: Pistos (irc.freenode.net)
-# http://purepistos.net/diakonos
-# Copyright (c) 2004-2009 Pistos
+# http://diakonos.pist0s.ca/
+# Copyright (c) 2004-2011 Pistos
 #
 # This software is released under the MIT licence.
 # See the LICENCE file included with this program, or
@@ -83,6 +83,7 @@ require 'diakonos/buffer/selection'
 require 'diakonos/buffer/undo'
 require 'diakonos/clipboard'
 require 'diakonos/clipboard-klipper'
+require 'diakonos/clipboard-klipper-dbus'
 require 'diakonos/clipboard-xclip'
 require 'diakonos/extension'
 require 'diakonos/extension-set'
@@ -102,6 +103,10 @@ module Diakonos
   DONT_PROMPT_OVERWRITE = false
   QUIET                 = true
   NOISY                 = false
+  AFTER_CHAR            = true
+  ON_CHAR               = false
+  INCLUSIVE             = true
+  NOT_INCLUSIVE         = false
 
   DEFAULT_TAB_SIZE = 8
 
@@ -152,18 +157,17 @@ module Diakonos
       @win_status       = nil
       @win_interaction  = nil
       @win_line_numbers = nil
-      @buffers = BufferHash.new
+      @buffers          = Array.new
 
       load_configuration
 
-      @quitting    = false
-      @untitled_id = 0
+      @quitting         = false
 
       @x = 0
       @y = 0
 
       @buffer_stack           = Array.new
-      @current_buffer         = nil
+      @buffer_current         = nil
 
       @cursor_stack           = Array.new
       @cursor_stack_pointer   = nil
@@ -247,7 +251,9 @@ module Diakonos
             print_usage
             exit 1
           else
-            @read_only_files.push session_file_hash_for( filename )
+            h = session_file_hash_for( filename )
+            h[ 'read_only' ] = true
+            @read_only_files.push h
           end
         when '-s', '--load-session'
           @session_to_load = session_filepath_for( argv.shift )
@@ -266,7 +272,6 @@ module Diakonos
         end
       end
     end
-    protected :parse_options
 
     def print_usage
       puts "Usage: #{$0} [options] [file] [file...]"
@@ -278,7 +283,6 @@ module Diakonos
       puts "\t-s, --load-session <session identifier>\tLoad a session"
       puts "\t--uninstall[-without-confirmation]\tUninstall Diakonos"
     end
-    protected :print_usage
 
     def clear_non_movement_flag
       @there_was_non_movement = false
@@ -291,6 +295,28 @@ module Diakonos
 
       initialize_display
 
+      @files.each do |file|
+        @buffers << Buffer.new( file )
+      end
+      @files = []
+      @read_only_files.each do |file|
+        @buffers << Buffer.new( file )
+      end
+      if ! @testing
+        session_buffers = session_startup
+      end
+      session_buffer_number = @session[ 'buffer_current' ] || 1
+      @files.each do |file_info|
+        @buffers << Buffer.new( file_info )
+      end
+      if @buffers.empty?
+        @buffers << Buffer.new
+      end
+
+      @buffers.each do |buffer|
+        buffer.reset_display
+      end
+
       if ENV[ 'COLORTERM' ] == 'gnome-terminal'
         help_key = 'Shift-F1'
       else
@@ -298,9 +324,6 @@ module Diakonos
       end
       set_iline "Diakonos #{VERSION} (#{LAST_MODIFIED})   #{help_key} for help  F12 to configure  Ctrl-Q to quit"
 
-      session_buffers = session_startup
-      session_buffer_number = @session[ 'current_buffer' ] || 1
-
       scripts = @extensions.scripts + Dir[ "#{@script_dir}/*" ]
       scripts.each do |script|
         begin
@@ -320,62 +343,57 @@ module Diakonos
         hook.sort { |a,b| a[ :priority ] <=> b[ :priority ] }
       end
 
-      num_opened = 0
-      if @files.length == 0 && @read_only_files.length == 0
-        if open_file
-          num_opened += 1
-        end
-      else
-        @files.each do |file|
-          if open_file( file[ 'filepath' ], file )
-            num_opened += 1
-          end
-        end
-        @read_only_files.each do |file|
-          if open_file( file[ 'filepath' ], 'read_only' => true )
-            num_opened += 1
-          end
-        end
-      end
-
       if session_buffers
         session_buffers.each do |buffer|
           close_file buffer
         end
       end
 
-      if num_opened > 0
-        switch_to_buffer_number 1
-
-        update_status_line
-        update_context_line
-
-        if @post_load_script
-          eval @post_load_script
-        end
-
-        run_hook_procs :after_startup
-        switch_to_buffer_number session_buffer_number
-
-        if ! @settings[ 'suppress_welcome' ]
-          open_file "#{@help_dir}/welcome.dhf"
-        end
-
-        begin
-          # Main keyboard loop.
-          while ! @quitting
-            process_keystroke
-            @win_main.refresh
-          end
-        rescue SignalException => e
-          debug_log "Terminated by signal (#{e.message})"
-        end
-
-        cleanup_display
-        cleanup_session
-
-        @debug.close
+      if @post_load_script
+        eval @post_load_script
       end
+
+      run_hook_procs :after_startup
+      if ! switch_to_buffer_number( session_buffer_number )
+        debug_log "Failed to switch to buffer #{session_buffer_number.inspect}"
+        switch_to_buffer_number 1
+      end
+
+      @buffers.each do |b|
+        run_hook_procs :after_open, b
+        b.cursor_to( b.last_row, b.last_col, Buffer::DONT_DISPLAY )
+      end
+
+      if @config_problems.any?
+        File.open( @error_filename, "w" ) do |f|
+          f.puts "There are #{@config_problems.size} problems with the configuration file(s):"
+          @config_problems.each do |p|
+            f.puts p
+          end
+        end
+        open_file @error_filename
+      end
+
+      if ! @settings[ 'suppress_welcome' ]
+        open_file "#{@help_dir}/welcome.dhf"
+      else
+        @buffer_current.seek /<<<</
+      end
+
+      begin
+        # Main keyboard loop.
+        while ! @quitting
+          process_keystroke
+          @win_main.refresh
+        end
+      rescue SignalException => e
+        debug_log "Terminated by signal (#{e.message})"
+      end
+
+      cleanup_display
+      cleanup_session
+
+      @debug.close
     end
 
     def uninstall( confirm = true )
@@ -451,7 +469,7 @@ module Diakonos
           f.puts
           f.puts "----------------------------------------------------"
           f.puts "If you can reproduce this error, please report it at"
-          f.puts "http://linis.purepistos.net/ticket/list/Diakonos !"
+          f.puts "https://github.com/Pistos/diakonos/issues !"
           f.puts "----------------------------------------------------"
           f.puts e.backtrace
         end
@@ -469,7 +487,6 @@ module Diakonos
       @macro_input_history = Array.new
       set_iline "Started macro recording."
     end
-    protected :start_recording_macro
 
     def stop_recording_macro
       @macro_history.pop  # Remove the stop_recording_macro command itself
@@ -478,12 +495,11 @@ module Diakonos
       @macro_input_history = nil
       set_iline "Stopped macro recording."
     end
-    protected :stop_recording_macro
 
     def load_tags
       @tags = Hash.new
-      if @current_buffer && @current_buffer.name
-        path = File.expand_path( File.dirname( @current_buffer.name ) )
+      if buffer_current && buffer_current.name
+        path = File.expand_path( File.dirname( buffer_current.name ) )
         tagfile = path + "/tags"
       else
         tagfile = "./tags"
@@ -505,10 +521,6 @@ module Diakonos
       end
     end
 
-    def settings
-      @settings.merge @session[ 'settings' ]
-    end
-
     def escape_quotes( str )
       temp = ''
       str.each_byte do |b|
@@ -539,6 +551,24 @@ module Diakonos
       end
     end
 
+    # @return [Array] the filename and line number parsed
+    def parse_filename_and_line_number( s )
+      if(
+        # Ruby
+        s =~ /from (.+):(\d+)/ ||
+        # Python
+        s =~ /File "(.+)", line (\d+)/ ||
+        # Perl
+        s =~ /at (.+) line (\d+)/ ||
+        # generic
+        s =~ /^(.+):(\d+)/
+      )
+        [ $1, ( $2.to_i - 1 ) ]
+      else
+        [ s, nil ]
+      end
+    end
+
   end
 
 end
