@@ -11,7 +11,7 @@ module Diakonos
   EOL_ALT_LAST_CHAR = 3
 
   class Diakonos
-    attr_reader :token_regexps, :close_token_regexps, :token_formats, :diakonos_conf, :column_markers, :surround_pairs
+    attr_reader :token_regexps, :close_token_regexps, :token_formats, :diakonos_conf, :column_markers, :surround_pairs, :settings
 
     def fetch_conf( location = "v#{VERSION}" )
       require 'open-uri'
@@ -90,12 +90,13 @@ module Diakonos
       @token_formats       = Hash.new { |h,k| h[ k ] = Hash.new }
       @column_markers      = Hash.new { |h,k| h[ k ] = Hash.new }
       @indenters           = Hash.new
-      @indenters_next_line  = Hash.new
+      @indenters_next_line = Hash.new
       @unindenters         = Hash.new
       @filemasks           = Hash.new
       @bangmasks           = Hash.new
       @closers             = Hash.new
       @surround_pairs      = Hash.new { |h,k| h[ k ] = Hash.new}
+      @fuzzy_ignores       = Array.new
 
       @settings = Hash.new
       @setting_strings = Hash.new
@@ -108,18 +109,27 @@ module Diakonos
       @colour_pairs = Array.new
 
       begin
+        @configs = []
+        @config_problems = []
         parse_configuration_file @global_diakonos_conf
         parse_configuration_file @diakonos_conf
 
-        # Session settings override config file settings.
+        languages = @surround_pairs.keys | @token_regexps.keys | @close_token_regexps.keys | @token_formats.keys
 
-        @session[ 'settings' ].each do |key,value|
-          @settings[ key ] = value
+        languages.each do |language|
+          @surround_pairs[ language ] = @surround_pairs[ 'all' ].merge( @surround_pairs[ language ] )
+          @token_regexps[ language ] = @token_regexps[ 'all' ].merge( @token_regexps[ language ] )
+          @close_token_regexps[ language ] = @close_token_regexps[ 'all' ].merge( @close_token_regexps[ language ] )
+          @token_formats[ language ] = @token_formats[ 'all' ].merge( @token_formats[ language ] )
         end
 
+        merge_session_settings
+
         case @settings[ 'clipboard.external' ]
-        when 'klipper'
+        when 'klipper', 'klipper-dcop'
           @clipboard = ClipboardKlipper.new
+        when 'klipper-dbus'
+          @clipboard = ClipboardKlipperDBus.new
         when 'xclip'
           @clipboard = ClipboardXClip.new
         else
@@ -128,7 +138,7 @@ module Diakonos
         @log = File.open( @logfilename, "a" )
 
         if @buffers
-          @buffers.each_value do |buffer|
+          @buffers.each do |buffer|
             buffer.configure
           end
         end
@@ -177,10 +187,16 @@ module Diakonos
       end
     end
 
-    def parse_configuration_file( filename )
+    # @param [String] filename_ the config file to parse
+    # @param [String] including_filename the config file which calls include on this one
+    # @return an Array of problem descriptions (Strings)
+    def parse_configuration_file( filename_, including_filename = nil )
+      filename = File.realpath( filename_ )
+      return  if @configs.any? { |c| c[:filename] == filename }
+      @configs << { filename: filename, source: including_filename }
       return  if ! FileTest.exists? filename
 
-      IO.foreach( filename ) do |line|
+      IO.readlines( filename ).each_with_index do |line,line_number|
         line.chomp!
         # Skip comments
         next  if line[ 0 ] == ?#
@@ -193,14 +209,19 @@ module Diakonos
         if arg.nil?
           command, arg = line.split( /\s+/, 2 )
           next  if command.nil?
+          if arg.nil?
+            @config_problems << "Configuration file '#{filename}' has an error on line #{line_number+1}"
+            next
+          end
         end
+
         command = command.downcase
 
         @setting_strings[ command ] = arg
 
         case command
         when "include"
-          parse_configuration_file File.expand_path( arg )
+          parse_configuration_file( File.expand_path( arg ), filename )
         when 'load_extension'
           @extensions.load( arg ).each do |conf_file|
             parse_configuration_file conf_file
@@ -315,23 +336,24 @@ module Diakonos
           @column_markers[ $1 ][ :format ] = Display.to_formatting( arg )
         when "logfile"
           @logfilename = File.expand_path( arg )
-        when "context.separator", "status.left", "status.right", "status.filler",
-            "status.modified_str", "status.unnamed_str", "status.selecting_str",
-            "status.read_only_str", /^lang\..+?\.indent\.ignore\.charset$/,
+        when "context.separator", /^lang\..+?\.indent\.ignore\.charset$/,
             /^lang\.(.+?)\.tokens\.([^.]+)\.change_to$/,
             /^lang\.(.+?)\.column_delimiters$/,
             "view.nonfilelines.character",
-            'interaction.blink_string', 'diff_command', 'session.default_session',
+            'diff_command', 'session.default_session',
             'clipboard.external'
           @settings[ command ] = arg
-        when /^lang\..+?\.comment_(?:close_)?string$/, 'view.line_numbers.number_format'
+        when /^lang\..+?\.comment_(?:close_)?string$/, 'view.line_numbers.number_format',
+            "status.filler", "status.left", "status.right",
+            "status.modified_str", "status.unnamed_str", "status.selecting_str",
+            "status.read_only_str", 'interaction.blink_string'
           @settings[ command ] = arg.gsub( /^["']|["']$/, '' )
         when "status.vars"
           @settings[ command ] = arg.split( /\s+/ )
         when /^lang\.(.+?)\.indent\.size$/, /^lang\.(.+?)\.(?:tabsize|wrap_margin)$/,
             "context.max_levels", "context.max_segment_width", "max_clips", "max_undo_lines",
             "view.margin.x", "view.margin.y", "view.scroll_amount", "view.lookback", 'grep.context',
-            'view.line_numbers.width'
+            'view.line_numbers.width', 'fuzzy_file_find.max_files'
           @settings[ command ] = arg.to_i
         when "view.jump.x", "view.jump.y"
           @settings[ command ] = [ arg.to_i, 1 ].max
@@ -361,6 +383,8 @@ module Diakonos
           end
         when "context.delay", 'interaction.blink_duration', 'interaction.choice_delay'
           @settings[ command ] = arg.to_f
+        when 'fuzzy_file_find.ignore'
+          @fuzzy_ignores << arg
         end
       end
     end
