@@ -1,6 +1,6 @@
 module Diakonos
-  VERSION       = '0.8.12'
-  LAST_MODIFIED = 'July 26, 2009'
+  VERSION       = '0.8.14'
+  LAST_MODIFIED = 'May 9, 2011'
 
   def self.parse_version( s )
     if s
@@ -18,4 +18,4 @@ module Diakonos
       exit 1
     end
   end
-end
\ No newline at end of file
+end
