@@ -13,6 +13,9 @@ end
 if ! Object.const_defined? 'SAMPLE_FILE'
   SAMPLE_FILE = File.join( TEST_DIR, '/sample-file.rb' )
 end
+if ! Object.const_defined? 'SAMPLE_FILE_LONGER'
+  SAMPLE_FILE_LONGER = File.join( TEST_DIR, '/longer-sample-file.rb' )
+end
 if ! Object.const_defined? 'SAMPLE_FILE_C'
   SAMPLE_FILE_C = File.join( TEST_DIR, '/sample-file.c' )
 end
@@ -31,6 +34,12 @@ def cursor_should_be_at( row, col )
   @b.current_column.should.equal col
 end
 
+def numbered_buffer_should_be_named( number, name_expected )
+  name = File.basename( @d.buffer_number_to_name( number ) )
+  name.should.equal name_expected
+end
+
+
 if $diakonos.nil?
   $diakonos = Diakonos::Diakonos.new [ '-e', 'quit', '--test', ]
   $diakonos.start
